
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.047.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28",
    "perl" => "5.013002"
  },
  "dist_abstract" => "Radis is a graylog logging radio through a redis database",
  "dist_author" => [
    "David Zurborg <zurborg\@cpan.org>"
  ],
  "dist_name" => "Log-Radis",
  "dist_version" => "0.002",
  "license" => "open_source",
  "module_name" => "Log::Radis",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "JSON" => "2.90",
    "Moo" => 2,
    "Redis" => "1.980",
    "Scalar::Util" => 0,
    "Sys::Hostname" => 0,
    "Time::HiRes" => "1.9726",
    "perl" => "5.013002",
    "strictures" => 2
  },
  "test_requires" => {
    "Test::Exception" => "0.40",
    "Test::Mock::Redis" => "0.18",
    "Test::More" => "1.001",
    "perl" => "5.013002"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Test::Exception" => "0.40",
  "Test::Mock::Redis" => "0.18",
  "Test::More" => "1.001",
  "perl" => "5.013002"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
